@@ -7,15 +7,13 @@ import os
 import logging
 import argparse
 import functools
-import re
-
-# there's a problem with functools32 and jsonschema. This import raise an
-# import error.
-#import jsonschema
 
+from .common import (SymbolTypes, validate_task_id,
+                     PacBioFloatOption, PacBioStringOption,
+                     PacBioBooleanOption, PacBioIntOption,
+                     PacBioStringChoiceOption, PacBioFloatChoiceOption, PacBioIntChoiceOption)
 from pbcommand.common_options import (add_base_options_with_emit_tool_contract,
                                       add_subcomponent_versions_option)
-from pbcommand.models import SymbolTypes
 from .tool_contract import (ToolDriver,
                             InputFileType, OutputFileType,
                             ToolContract, ToolContractTask,
@@ -32,30 +30,11 @@ __all__ = ["PbParser",
            "get_scatter_pbparser",
            "get_gather_pbparser"]
 
-RX_TASK_ID = re.compile(r'^([A-z0-9_]*)\.tasks\.([A-z0-9_]*)$')
-RX_TASK_OPTION_ID = re.compile(r'^([A-z0-9_]*)\.task_options\.([A-z0-9_\.]*)')
-
 
 def _to_file_type(format_):
     return "pacbio.file_types.{x}".format(x=format_)
 
 
-class JsonSchemaTypes(object):
-    # array is a native type, but not supported
-    BOOL = "boolean"
-    INT = "integer"
-    NUM = "number"
-    STR = "string"
-    NULL = "null"
-    OBJ = "object"
-
-    # Optional values e.g., Option[String]
-    OPT_BOOL = [BOOL, NULL]
-    OPT_INT = [INT, NULL]
-    OPT_STR = [STR, NULL]
-    OPT_NUM = [NUM, NULL]
-
-
 def _validate_file(label, path):
     if os.path.exists(path):
         return os.path.abspath(path)
@@ -77,6 +56,10 @@ def _validate_option_or_cast(dtype, dvalue):
                         "expected".format(a=dvalue, e=dtype))
 
 
+def to_opt_id(namespace, s):
+    return ".".join([namespace, "options", s])
+
+
 def _validate_option(dtype, dvalue):
     if isinstance(dvalue, dtype):
         return dvalue
@@ -85,92 +68,12 @@ def _validate_option(dtype, dvalue):
                         "expected".format(a=dvalue, e=dtype))
 
 
-def _validate_id(prog, idtype, tid):
-    if prog.match(tid):
-        return tid
-    else:
-        raise ValueError("Invalid format {t}: '{i}' {p}".format(t=idtype, i=tid, p=repr(prog.pattern)))
-
-_validate_task_id = functools.partial(_validate_id, RX_TASK_ID, 'task id')
-_validate_task_option_id = functools.partial(_validate_id, RX_TASK_OPTION_ID,
-                                             'task option id')
-
-
-def to_opt_id(namespace, s):
-    return ".".join([namespace, "options", s])
-
-
-def validate_value(schema, v):
-    import jsonschema
-    return jsonschema.validate(v, schema)
-
-
-def is_valid(schema, v):
-    """Returns a bool if the schema is valid"""
-    import jsonschema
-    try:
-        validate_value(schema, v)
-        return True
-    except jsonschema.ValidationError:
-        pass
-    return False
-
-
-def validate_schema(f):
-    """Deco for validate the returned jsonschema against Draft 4 of the spec"""
-    def w(*args, **kwargs):
-        schema = f(*args, **kwargs)
-        import jsonschema
-        _ = jsonschema.Draft4Validator(schema)
-        return schema
-    return w
-
-
-def to_option_schema(option_id, dtype_or_dtypes, display_name, description, default_value):
-    """
-    Simple util factory method
-    :param dtype_or_dtypes: single data type or list of data types
-    :param option_id: globally unique task option id. Must begin with
-    'pbsmrtpipe.task_options.'
-    :param display_name: display name of task options
-    :param description: Short description of the task options
-    :param required: Is the option required.
-    """
-    # annoying that you can't specify a tuple
-    if isinstance(dtype_or_dtypes, tuple):
-        dtype_or_dtypes = list(dtype_or_dtypes)
-
-    _validate_task_option_id(option_id)
-
-    # Steps toward moving away from JSON schema as the format, but reuse
-    # the jsonschema defined types. Only non-union types are supported.
-    pbd = {"option_id": option_id,
-           "type": dtype_or_dtypes,
-           "default": default_value,
-           "name": display_name,
-           "description": description}
-
-    d = {'$schema': "http://json-schema.org/draft-04/schema#",
-         'type': 'object',
-         'title': "JSON Schema for {o}".format(o=option_id),
-         'properties': {option_id: {'description': description,
-                                    'title': display_name,
-                                    'type': dtype_or_dtypes},
-                        },
-         "pb_option": pbd
-         }
-
-    d['required'] = [option_id]
-    d['properties'][option_id]['default'] = default_value
-    return d
-
-
 class PbParserBase(object):
 
     __metaclass__ = abc.ABCMeta
 
     def __init__(self, tool_id, version, name, description):
-        self.tool_id = _validate_task_id(tool_id)
+        self.tool_id = validate_task_id(tool_id)
         self.version = version
         self.description = description
         self.name = name
@@ -273,6 +176,55 @@ class PbParserBase(object):
         """
         raise NotImplementedError
 
+    @abc.abstractmethod
+    def add_choice_str(self, option_id, option_str, choices, name, description, default=None):
+        """
+        Add a generic enumerated argument whose type is a string.
+
+        :param option_id: fully-qualified option name used in tool contract
+                          layer, of form "pbcommand.task_options.my_option"
+        :param option_str: shorter parameter name, mainly used in Python
+                           argparse layer, but *without* leading dashes
+        :param choices: allowed values
+        :param name: plain-English name
+        :param description: help string
+        :param default: default value (if None, will use first choice)
+        """
+        raise NotImplementedError
+
+    @abc.abstractmethod
+    def add_choice_int(self, option_id, option_str, choices, name, description, default=None):
+        """
+        Add a generic enumerated argument whose type is an integer.
+
+        :param option_id: fully-qualified option name used in tool contract
+                          layer, of form "pbcommand.task_options.my_option"
+        :param option_str: shorter parameter name, mainly used in Python
+                           argparse layer, but *without* leading dashes
+        :param choices: allowed values
+        :param name: plain-English name
+        :param description: help string
+        :param default: default value (if None, will use first choice)
+        """
+        raise NotImplementedError
+
+    @abc.abstractmethod
+    def add_choice_float(self, option_id, option_str, choices, name, description, default=None):
+        """
+        Add a generic enumerated argument whose type is a float.
+
+        :param option_id: fully-qualified option name used in tool contract
+                          layer, of form "pbcommand.task_options.my_option"
+        :param option_str: shorter parameter name, mainly used in Python
+                           argparse layer, but *without* leading dashes
+        :param choices: allowed values
+        :param name: plain-English name
+        :param description: help string
+        :param default: default value (if None, will use first choice)
+        """
+        raise NotImplementedError
+
+
 _validate_argparse_int = functools.partial(_validate_option_or_cast, int)
 _validate_argparse_float = functools.partial(_validate_option_or_cast, float)
 _validate_argparse_bool = functools.partial(_validate_option_or_cast, bool)
@@ -284,10 +236,15 @@ class PyParser(PbParserBase):
 
     def __init__(self, tool_id, version, name, description, subcomponents=()):
         super(PyParser, self).__init__(tool_id, version, name, description)
-        self.parser = argparse.ArgumentParser(version=version,
-                                              description=description,
-                                              formatter_class=argparse.ArgumentDefaultsHelpFormatter,
-                                              add_help=True)
+        self.parser = argparse.ArgumentParser(  # version=version,
+            description=description,
+            formatter_class=argparse.ArgumentDefaultsHelpFormatter,
+            add_help=True)
+        self.parser.version = version
+        self.parser.add_argument('--version',
+                                 action="version",
+                                 help="show program's version number and exit")
+
         if subcomponents:
             add_subcomponent_versions_option(self.parser, subcomponents)
 
@@ -335,6 +292,27 @@ class PyParser(PbParserBase):
         self.parser.add_argument(opt, action=d[_validate_argparse_bool(not default)],
                                  help=description)
 
+    def _add_choice_base(self, opt_type, option_id, option_str, choices, name,
+                         description, default=None):
+        if default is None:
+            default = choices[0]
+        opt = '--' + option_str
+        self.parser.add_argument(opt, action="store", choices=choices,
+                                 type=opt_type,
+                                 help=description, default=default)
+
+    def add_choice_str(self, option_id, option_str, choices, name, description,
+                       default=None):
+        return self._add_choice_base(str, option_id, option_str, choices, name, description, default)
+
+    def add_choice_int(self, option_id, option_str, choices, name, description,
+                       default=None):
+        return self._add_choice_base(int, option_id, option_str, choices, name, description, default)
+
+    def add_choice_float(self, option_id, option_str, choices, name, description,
+                         default=None):
+        return self._add_choice_base(float, option_id, option_str, choices, name, description, default)
+
 
 class ToolContractParser(PbParserBase):
     """Parser to support Emitting and running ToolContracts"""
@@ -346,6 +324,7 @@ class ToolContractParser(PbParserBase):
         super(ToolContractParser, self).__init__(tool_id, version, name, description)
         self.input_types = []
         self.output_types = []
+        # List of PacBioOption and subclasses of PacBioOption
         self.options = []
         self.driver = driver
         self.name = name
@@ -362,26 +341,39 @@ class ToolContractParser(PbParserBase):
         self.output_types.append(x)
 
     def add_int(self, option_id, option_str, default, name, description):
-        self.options.append(to_option_schema(option_id,
-                                             JsonSchemaTypes.INT, name, description,
-                                             _validate_option(int, default)))
+        self.options.append(PacBioIntOption(option_id, name, default, description))
 
     def add_float(self, option_id, option_str, default, name, description):
         if isinstance(default, int):
             default = float(default)
-        self.options.append(to_option_schema(option_id,
-                                             JsonSchemaTypes.NUM, name, description,
-                                             _validate_option(float, default)))
+        self.options.append(PacBioFloatOption(option_id, name, default, description))
 
     def add_str(self, option_id, option_str, default, name, description):
-        self.options.append(to_option_schema(option_id,
-                                             JsonSchemaTypes.STR, name, description,
-                                             _validate_option(str, default)))
+        self.options.append(PacBioStringOption(option_id, name, default, description))
 
     def add_boolean(self, option_id, option_str, default, name, description):
-        self.options.append(to_option_schema(option_id,
-                                             JsonSchemaTypes.BOOL, name, description,
-                                             _validate_option(bool, default)))
+        self.options.append(PacBioBooleanOption(option_id, name, default, description))
+
+    def add_choice_str(self, option_id, option_str, choices, name, description,
+                       default=None):
+        if default is None:
+            default = choices[0]
+
+        self.options.append(PacBioStringChoiceOption(option_id, name, default, description, choices))
+
+    def add_choice_int(self, option_id, option_str, choices, name, description,
+                       default=None):
+        if default is None:
+            default = choices[0]
+
+        self.options.append(PacBioIntChoiceOption(option_id, name, default, description, choices))
+
+    def add_choice_float(self, option_id, option_str, choices, name, description,
+                         default=None):
+        if default is None:
+            default = choices[0]
+
+        self.options.append(PacBioFloatChoiceOption(option_id, name, default, description, choices))
 
     def to_tool_contract(self):
         # Not a well formed tool contract, must have at least one input and
@@ -405,6 +397,7 @@ class ToolContractParser(PbParserBase):
 
 
 class ScatterToolContractParser(ToolContractParser):
+
     def __init__(self, tool_id, version, name, description, task_type, driver, nproc_symbol,
                  resource_types, chunk_keys, nchunks):
         super(ScatterToolContractParser, self).__init__(tool_id, version, name, description, task_type, driver,
@@ -452,7 +445,7 @@ class PbParser(PbParserBase):
     (stored as tool_contract_parser and arg_parser attributes respectively).
     """
 
-    def __init__(self, tool_contract_parser, arg_parser, *parsers):
+    def __init__(self, tool_contract_parser, arg_parser, *parsers, **kwds):
         """
 
         :param tool_contract_parser:
@@ -469,7 +462,8 @@ class PbParser(PbParserBase):
         # python wrapper parser.
         self.arg_parser = arg_parser
         # add options, so it will show up via --help
-        add_base_options_with_emit_tool_contract(self.arg_parser.parser)
+        add_base_options_with_emit_tool_contract(self.arg_parser.parser,
+                                                 default_level=kwds.get("default_level", "INFO"))
 
         # a list of other parsers that adhere to the PbParserBase interface
         # can be used.
@@ -516,19 +510,35 @@ class PbParser(PbParserBase):
         args = option_id, option_str, default, name, description
         self._dispatch("add_boolean", args, {})
 
+    def add_choice_str(self, option_id, option_str, choices, name, description,
+                       default=None):
+        args = option_id, option_str, choices, name, description, default
+        self._dispatch("add_choice_str", args, {})
+
+    def add_choice_int(self, option_id, option_str, choices, name, description,
+                       default=None):
+        args = option_id, option_str, choices, name, description, default
+        self._dispatch("add_choice_int", args, {})
+
+    def add_choice_float(self, option_id, option_str, choices, name, description,
+                         default=None):
+        args = option_id, option_str, choices, name, description, default
+        self._dispatch("add_choice_float", args, {})
+
     def to_contract(self):
         return self.tool_contract_parser.to_tool_contract()
 
 
-def _factory(tool_id, version, name, description, subcomponents):
+def _factory(tool_id, version, name, description, subcomponents, default_level):
     def _f(tc_parser):
         arg_parser = PyParser(tool_id, version, name, description, subcomponents=subcomponents)
-        return PbParser(tc_parser, arg_parser)
+        return PbParser(tc_parser, arg_parser, default_level=default_level)
     return _f
 
 
 def get_pbparser(tool_id, version, name, description, driver_exe, is_distributed=True, nproc=1,
-                 resource_types=(), subcomponents=(), serialization='json'):
+                 resource_types=(), subcomponents=(), serialization='json',
+                 default_level="INFO"):
     """
     Central point of creating a Tool contract that can emit and run tool
     contracts.
@@ -538,24 +548,24 @@ def get_pbparser(tool_id, version, name, description, driver_exe, is_distributed
     driver = ToolDriver(driver_exe, serialization=serialization)
     tc_parser = ToolContractParser(tool_id, version, name, description, is_distributed, driver,
                                    nproc, resource_types)
-    return _factory(tool_id, version, name, description, subcomponents)(tc_parser)
+    return _factory(tool_id, version, name, description, subcomponents, default_level)(tc_parser)
 
 
 def get_scatter_pbparser(tool_id, version, name, description, driver_exe, chunk_keys,
                          is_distributed=True, nproc=1, nchunks=SymbolTypes.MAX_NCHUNKS, resource_types=(),
-                         subcomponents=(), serialization='json'):
+                         subcomponents=(), serialization='json', default_level="INFO"):
     """Create a Scatter Tool"""
     driver = ToolDriver(driver_exe, serialization=serialization)
     tc_parser = ScatterToolContractParser(tool_id, version, name, description, is_distributed,
                                           driver, nproc, resource_types, chunk_keys,
                                           nchunks)
-    return _factory(tool_id, version, name, description, subcomponents)(tc_parser)
+    return _factory(tool_id, version, name, description, subcomponents, default_level)(tc_parser)
 
 
 def get_gather_pbparser(tool_id, version, name, description, driver_exe,
-                        is_distributed=True, nproc=1, resource_types=(), subcomponents=(), serialization='json'):
+                        is_distributed=True, nproc=1, resource_types=(), subcomponents=(), serialization='json', default_level="INFO"):
     """Create a Gather tool"""
     driver = ToolDriver(driver_exe, serialization=serialization)
     tc_parser = GatherToolContractParser(tool_id, version, name, description,
                                          is_distributed, driver, nproc, resource_types)
-    return _factory(tool_id, version, name, description, subcomponents)(tc_parser)
+    return _factory(tool_id, version, name, description, subcomponents, default_level)(tc_parser)
