@@ -5,6 +5,7 @@ import json
 from pprint import pformat
 
 from pbcommand.pb_io import load_report_from_json
+from pbcommand.models.report import Plot, PlotlyPlot
 
 _SERIALIZED_JSON_DIR = 'example-reports'
 
@@ -30,6 +31,12 @@ class TestSerializationOverviewReport(unittest.TestCase):
     def test_id(self):
         self.assertEqual(self.report.id, "overview")
 
+    def test_uuid(self):
+        self.assertEqual(self.report.uuid, "196136c8-f6fd-11e5-b481-3c15c2cc8f88")
+
+    def test_title(self):
+        self.assertEqual(self.report.title, "Overview Report")
+
     def test_attributes(self):
         self.assertTrue(len(self.report.attributes), 2)
 
@@ -48,7 +55,10 @@ class TestSerializationAdapterReport(unittest.TestCase):
         self.assertEqual(len(self.report.attributes), 6)
 
     def test_plotgroups(self):
-        self.assertEqual(len(self.report.plotGroups), 1)
+        self.assertEqual(len(self.report.plotGroups), 2)
 
     def test_plots(self):
         self.assertEqual(len(self.report.plotGroups[0].plots), 1)
+        self.assertEqual(len(self.report.plotGroups[1].plots), 1)
+        self.assertEqual(self.report.plotGroups[0].plots[0].plotType, Plot.PLOT_TYPE)
+        self.assertEqual(self.report.plotGroups[1].plots[0].plotType, PlotlyPlot.PLOT_TYPE)
